ATLAS = {
    # From https://github.com/kratsg/ATLASstylempl
    "lines.linewidth": 1,
    "font.family": "sans-serif",
    "font.sans-serif": [
        "helvetica",
        "Helvetica",
        "Nimbus Sans L",
        "Mukti Narrow",
        "FreeSans",
    ],
    "font.size": 18,
    "mathtext.fontset": "stixsans",
    "mathtext.default": "rm",
    # figure layout
    "figure.figsize": (8.75, 5.92),
    "figure.facecolor": "white",
    "figure.subplot.bottom": 0.16,
    "figure.subplot.top": 0.93,
    "figure.subplot.left": 0.16,
    "figure.subplot.right": 0.95,
    # axes
    "axes.labelsize": 30,
    "axes.labelpad": 30,
    "xtick.top": True,
    "xtick.labelsize": 19,
    "xtick.major.size": 10,
    "xtick.minor.size": 5,
    "xtick.direction": "in",
    "xtick.minor.visible": True,
    "ytick.right": True,
    "ytick.labelsize": 19,
    "ytick.major.size": 14,
    "ytick.minor.size": 7,
    "ytick.direction": "in",
    "ytick.minor.visible": True,
    "lines.markersize": 8,
    # legend
    "legend.numpoints": 1,
    "legend.fontsize": 18,
    "legend.labelspacing": 0.3,
    "legend.frameon": False,
}
